# coding=utf-8
import logging
import os
import sys
from datetime import datetime
from logging import Logger, StreamHandler

import json_logging
import inspect


def is_env_var_toggle(var_name):
    enable_json_setting = str(os.getenv(var_name)).lower()
    _env_toggle = enable_json_setting.lower() in ['true', '1', 'y', 'yes']
    return _env_toggle


def get_library_logger(logger_name):
    """

    :param logger_name: name
    :return: logger
    """
    # noinspection PyUnresolvedReferences
    if logger_name in logging.Logger.manager.loggerDict:
        return logging.getLogger(logger_name)

    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.DEBUG)
    # add stdout output in case parent have no handlers
    if len(logger.parent.handlers) == 0:
        logger.addHandler(StreamHandler(sys.stdout))

    return logger


def update_formatter_for_loggers(loggers_iter, formatter):
    """
    :param formatter:
    :param loggers_iter:
    """
    for logger in loggers_iter:
        if not isinstance(logger, Logger):
            raise RuntimeError("%s is not a logging.Logger instance", logger)
        for handler in logger.handlers:
            if not isinstance(handler.formatter, formatter):
                handler.formatter = formatter()


# noinspection PyPep8
def parse_int(input_int, default):
    # noinspection PyBroadException
    try:
        integer = int(input_int)
    except:
        integer = default
    return integer


def validate_subclass(subclass, superclass):
    """

    :param subclass
    :param superclass
    :return: bool
    """
    if not issubclass(subclass, superclass):
        raise RuntimeError(str(subclass) + ' is not a subclass of ' + str(superclass))

    return True


_epoch = datetime(1970, 1, 1)


def epoch_nano_second(datetime_):
    return int((datetime_ - _epoch).total_seconds()) * 1000000000 + datetime_.microsecond * 1000


def iso_time_format(datetime_):
    return '%04d-%02d-%02dT%02d:%02d:%02d.%03dZ' % (
        datetime_.year, datetime_.month, datetime_.day, datetime_.hour, datetime_.minute, datetime_.second,
        int(datetime_.microsecond / 1000))


if hasattr(sys, '_getframe'):
    currentframe = lambda _no_of_go_up_level: sys._getframe(_no_of_go_up_level)
else:  # pragma: no cover
    # noinspection PyBroadException
    def currentframe(_no_of_go_up_level):
        """Return the frame object for the caller's stack frame."""
        try:
            raise Exception
        except Exception:
            return sys.exc_info()[_no_of_go_up_level - 1].tb_frame.f_back


class RequestUtil(object):
    """
        util for extract request's information
    """

    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            request_adapter_class = kw['request_adapter_class']
            response_adapter_class = kw['response_adapter_class']

            validate_subclass(request_adapter_class, json_logging.RequestAdapter)
            validate_subclass(response_adapter_class, json_logging.ResponseAdapter)

            cls._instance = object.__new__(cls)

            cls._instance.request_adapter_class = request_adapter_class
            cls._instance.response_adapter_class = response_adapter_class
            cls._instance.request_adapter = request_adapter_class()
            cls._instance.response_adapter = response_adapter_class()
            cls._instance.is_support_global_request_object = request_adapter_class.support_global_request_object()
            cls._instance.create_correlation_id_if_not_exists = json_logging.CREATE_CORRELATION_ID_IF_NOT_EXISTS

        return cls._instance

    def get_correlation_id(self, request=None,within_formatter=False):
        """
        Gets the correlation id from the header of the request. \
        It tries to search from json_logging.CORRELATION_ID_HEADERS list, one by one.\n
        If found no value, new id will be generated by default.\n
        :param request: request object
        :return: correlation id string
        """
        # _logger.debug("Getting correlation", extra={'correlation_id': '-'})
        #
        if request is None:
            if self.is_support_global_request_object:
                request = self.request_adapter_class.get_current_request()
            else:
                request = self.get_request_from_call_stack()

            if request is None:
                return json_logging.EMPTY_VALUE

        # _logger.debug("Attempt to get correlation from request context", extra={'correlation_id': '-'})
        correlation_id = self.request_adapter.get_correlation_id_in_request_context(request)
        if correlation_id is not None:
            return correlation_id

        correlation_id = self._get_correlation_id_in_request_header(self.request_adapter, request)
        # exists = json_logging.CREATE_CORRELATION_ID_IF_NOT_EXISTS
        if correlation_id is None and self.create_correlation_id_if_not_exists:
            correlation_id = str(json_logging.CORRELATION_ID_GENERATOR())
            self.request_adapter.set_correlation_id(request, correlation_id)

        return correlation_id if correlation_id else json_logging.EMPTY_VALUE

    def get_request_from_call_stack(self, within_formatter=False):
        """

        :return: get request object from call stack
        """
        """
            python 3 call stack frame
            00 get_request_from_call_stack [util.py:225]
            01 get_correlation_id [util.py:177]
            02 format [__init__.py:333]
            03 format [__init__.py:830]
            04 emit [__init__.py:980]
            05 handle [__init__.py:855]
            06 callHandlers [__init__.py:1487]
            07 handle [__init__.py:1425]
            08 _log [__init__.py:1415]
            09 info [__init__.py:1279]
            10 logging statement
        """
        module = inspect.getmodule(inspect.currentframe().f_back)

        class_type = self.request_adapter_class.get_request_class_type()
        no_of_go_up_level = 11 if within_formatter else 1

        # FIXME: find out the depth of logging call stack in Python 2.7
        f = currentframe(no_of_go_up_level)
        while True:
            f_locals = f.f_locals
            if 'request' in f_locals:
                if isinstance(f_locals['request'], class_type):
                    return f_locals['request']

            if 'req' in f_locals:
                if isinstance(f_locals['req'], class_type):
                    return f_locals['req']

            for key in f_locals:
                if key not in {'request', 'req'} and isinstance(f_locals[key], class_type):
                    return f_locals[key]
            if f.f_back is not None:
                f = f.f_back
            else:
                break
        return None

    @staticmethod
    def _get_correlation_id_in_request_header(request_adapter, request):
        for header in json_logging.CORRELATION_ID_HEADERS:
            value = request_adapter.get_http_header(request, header)
            if value is not None:
                return value
        return None



# coding=utf-8
import logging
import sys

import json_logging
import json_logging.framework
from json_logging import JSONLogWebFormatter
from json_logging.framework_base import AppRequestInstrumentationConfigurator, RequestAdapter, ResponseAdapter


def is_connexion_present():
    # noinspection PyPep8,PyBroadException
    try:
        import connexion
        return True
    except:
        return False


if is_connexion_present():
    from connexion import request as request_obj
    import connexion as connexion
    import flask as flask
    from flask import g

    _current_request = request_obj
    _connexion = connexion
    _flask = flask
    _connexion.g = g


class ConnexionAppRequestInstrumentationConfigurator(AppRequestInstrumentationConfigurator):
    def config(self, app):
        if not is_connexion_present():
            raise RuntimeError("connexion is not available in system runtime")
        from flask.app import Flask
        if not isinstance(app.app, Flask):
            raise RuntimeError("app is not a valid connexion.app.Connexion app instance")

        # Disable standard logging
        logging.getLogger('werkzeug').disabled = True

        json_logging.util.update_formatter_for_loggers([logging.getLogger('werkzeug')], JSONLogWebFormatter)

        # noinspection PyAttributeOutsideInit
        self.request_logger = logging.getLogger('connexion-request-logger')


        from flask import g

        @app.app.before_request
        def before_request():
            g.request_info = json_logging.RequestInfo(_current_request)

        @app.app.after_request
        def after_request(response):
            request_info = g.request_info
            request_info.update_response_status(response)
            self.request_logger.info("", extra={'request_info': request_info})
            return response


class ConnexionRequestAdapter(RequestAdapter):
    @staticmethod
    def get_request_class_type():
        raise NotImplementedError

    @staticmethod
    def support_global_request_object():
        return True

    @staticmethod
    def get_current_request():
        return _current_request

    def get_remote_user(self, request):
        if request.authorization is not None:
            return request.authorization.username
        else:
            return json_logging.EMPTY_VALUE

    def get_http_header(self, request, header_name, default=None):
        if header_name in request.headers:
            return request.headers.get(header_name)
        return default

    def set_correlation_id(self, request_, value):
        _connexion.g.correlation_id = value

    def get_correlation_id_in_request_context(self, request):
        return _connexion.g.get('correlation_id', None)

    def get_protocol(self, request):
        return request.environ.get('SERVER_PROTOCOL')

    def get_path(self, request):
        return request.path

    def get_content_length(self, request):
        return request.content_length

    def get_method(self, request):
        return request.method

    def get_remote_ip(self, request):
        return request.remote_addr

    def get_remote_port(self, request):
        return request.environ.get('REMOTE_PORT')


class ConnexionResponseAdapter(ResponseAdapter):
    def get_status_code(self, response):
        return response.status_code

    def get_response_size(self, response):
        return response.calculate_content_length()

    def get_content_type(self, response):
        return response.content_type



# coding=utf-8
import logging
import logging.config

import sys

import json_logging
import json_logging.framework
from json_logging.framework_base import FrameworkConfigurator, AppRequestInstrumentationConfigurator, RequestAdapter, \
    ResponseAdapter


def is_sanic_present():
    # noinspection PyBroadException
    try:
        # noinspection PyPackageRequirements
        from sanic import Sanic
        return True
    except:
        return False


class SanicAppConfigurator(FrameworkConfigurator):

    def config(self):
        if not is_sanic_present():
            raise RuntimeError("Sanic is not available in system runtime")

        # from sanic.config import LOGGING
        # noinspection PyPackageRequirements
        from sanic.log import LOGGING_CONFIG_DEFAULTS

        LOGGING_CONFIG_DEFAULTS['disable_existing_loggers'] = False
        LOGGING_CONFIG_DEFAULTS['formatters']['generic']['class'] = "json_logging.JSONLogFormatter"
        del LOGGING_CONFIG_DEFAULTS['formatters']['generic']['format']

        LOGGING_CONFIG_DEFAULTS['formatters']['access']['class'] = "json_logging.JSONLogFormatter"
        del LOGGING_CONFIG_DEFAULTS['formatters']['access']['format']

        # logging.config.dictConfig(LOGGING_CONFIG_DEFAULTS)


class SanicAppRequestInstrumentationConfigurator(AppRequestInstrumentationConfigurator):
    def config(self, app):
        if not is_sanic_present():
            raise RuntimeError("Sanic is not available in system runtime")
        # noinspection PyPackageRequirements
        from sanic import Sanic
        if not isinstance(app, Sanic):
            raise RuntimeError("app is not a valid Sanic.app.Sanic app instance")

        # noinspection PyAttributeOutsideInit
        self.request_logger = logging.getLogger('sanic-request')

        logging.getLogger('sanic.access').disabled = True

        @app.middleware('request')
        def before_request(request):
            request.ctx.request_info = json_logging.RequestInfo(request)

        @app.middleware('response')
        def after_request(request, response):
            request_info = request.ctx.request_info
            request_info.update_response_status(response)
            self.request_logger.info("", extra={'request_info': request_info, 'type': 'request'})

    def get_request_logger(self):
        return self.request_logger


class SanicRequestAdapter(RequestAdapter):

    @staticmethod
    def get_current_request():
        raise NotImplementedError

    @staticmethod
    def support_global_request_object():
        return False

    # noinspection PyPackageRequirements
    @staticmethod
    def get_request_class_type():
        from sanic.request import Request
        return Request

    def get_remote_user(self, request):
        # todo: implement it
        return None

    def get_http_header(self, request, header_name, default=None):
        if header_name in request.headers:
            return request.headers.get(header_name)
        return default

    def set_correlation_id(self, request, value):
        request.ctx.correlation_id = value

    def get_correlation_id_in_request_context(self, request):
        try:
            return request.ctx.correlation_id
        except AttributeError:
            return None

    def get_protocol(self, request):
        return json_logging.EMPTY_VALUE

    def get_path(self, request):
        return request.path

    def get_content_length(self, request):
        return json_logging.EMPTY_VALUE

    def get_method(self, request):
        return request.method

    def get_remote_ip(self, request):
        return request.ip[0]

    def get_remote_port(self, request):
        return json_logging.EMPTY_VALUE


class SanicResponseAdapter(ResponseAdapter):

    def get_status_code(self, response):
        return response.status

    def get_response_size(self, response):
        return json_logging.EMPTY_VALUE

    def get_content_type(self, response):
        return response.content_type



import io
import sys

from setuptools import setup, find_packages

version_info_major = sys.version_info[0]
if version_info_major == 3:
    long_description = open('README.rst', encoding="utf8").read()
else:
    io_open = io.open('README.rst', encoding="utf8")
    long_description = io_open.read()

setup(
    name="json-logging",
    version='1.2.4',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*', 'example', 'dist', 'build']),
    license='Apache License 2.0',
    description="JSON Python Logging",
    long_description=long_description,
    author="Bui Nguyen Thang (Bob)",
    author_email="bob.bui@outlook.com",
    keywords=["json", "elastic", "python", "python3", "python2", "logging", "logging-library", "json", "elasticsearch",
              "elk", "elk-stack", "logstash", "kibana"],
    platforms='any',
    url="https://github.com/thangbn/json-logging",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: System :: Logging',
        'Framework :: Flask',
    ]
)



# coding=utf-8
import json
import logging
import sys
import uuid
from datetime import datetime
import traceback

from json_logging import util
from json_logging.framework_base import RequestAdapter, ResponseAdapter, AppRequestInstrumentationConfigurator, \
    FrameworkConfigurator
from json_logging.util import get_library_logger, is_env_var_toggle

CORRELATION_ID_GENERATOR = uuid.uuid1
ENABLE_JSON_LOGGING = False
if is_env_var_toggle("ENABLE_JSON_LOGGING"):
    ENABLE_JSON_LOGGING = True

ENABLE_JSON_LOGGING_DEBUG = False
EMPTY_VALUE = '-'
CREATE_CORRELATION_ID_IF_NOT_EXISTS = True
JSON_SERIALIZER = lambda log: json.dumps(log, ensure_ascii=False)
CORRELATION_ID_HEADERS = ['X-Correlation-ID', 'X-Request-ID']
COMPONENT_ID = EMPTY_VALUE
COMPONENT_NAME = EMPTY_VALUE
COMPONENT_INSTANCE_INDEX = 0

_framework_support_map = {}
_current_framework = None
_logger = get_library_logger(__name__)
_request_util = None
_default_formatter = None


def get_correlation_id(request=None):
    """
    Get current request correlation-id. If one is not present, a new one might be generated
    depends on CREATE_CORRELATION_ID_IF_NOT_EXISTS setting value.

    :return: correlation-id string
    """
    return _request_util.get_correlation_id(request=request)


def register_framework_support(name, app_configurator, app_request_instrumentation_configurator, request_adapter_class,
                               response_adapter_class):
    """
    register support for a framework

    :param name: name of framework
    :param app_configurator: app pre-configurator class
    :param app_request_instrumentation_configurator: app configurator class
    :param request_adapter_class: request adapter class
    :param response_adapter_class: response adapter class
    """
    if not name:
        raise RuntimeError("framework name can not be null or empty")

    util.validate_subclass(request_adapter_class, RequestAdapter)
    util.validate_subclass(response_adapter_class, ResponseAdapter)
    util.validate_subclass(app_request_instrumentation_configurator, AppRequestInstrumentationConfigurator)
    if app_configurator is not None:
        util.validate_subclass(app_configurator, FrameworkConfigurator)

    name = name.lower()
    if name in _framework_support_map:
        ENABLE_JSON_LOGGING_DEBUG and _logger.warning("Re-register framework %s", name)
    _framework_support_map[name] = {
        'app_configurator': app_configurator,
        'app_request_instrumentation_configurator': app_request_instrumentation_configurator,
        'request_adapter_class': request_adapter_class,
        'response_adapter_class': response_adapter_class
    }


def config_root_logger():
    """
        You must call this if you are using root logger.
        Make all root logger' handlers produce JSON format
        & remove duplicate handlers for request instrumentation logging.
        Please made sure that you call this after you called "logging.basicConfig() or logging.getLogger('root')
    """
    global _default_formatter

    if len(logging.root.handlers) > 0:
        _logger.error(
            "No logging handlers found for root logger. Please made sure that you call this after you called "
            "logging.basicConfig() or logging.getLogger('root')")

    if ENABLE_JSON_LOGGING:
        ENABLE_JSON_LOGGING_DEBUG and _logger.debug("Update root logger to using JSONLogFormatter")

        util.update_formatter_for_loggers([logging.root], _default_formatter)


def init_non_web(*args, **kw):
    __init(*args, **kw)


def __init(framework_name=None, custom_formatter=None, enable_json=False):
    """
    Initialize JSON logging support, if no **framework_name** passed, logging will be initialized in non-web context.
    This is supposed to be called only one time.

    If **custom_formatter** is passed, it will (in non-web context) use this formatter over the default.

    :param framework_name: type of framework logging should support.
    :param custom_formatter: formatter to override default JSONLogFormatter.
    """

    global _current_framework
    global ENABLE_JSON_LOGGING
    global _default_formatter
    ENABLE_JSON_LOGGING = enable_json
    if _current_framework is not None:
        raise RuntimeError("Can not call init more than once")

    if custom_formatter:
        if not issubclass(custom_formatter, logging.Formatter):
            raise ValueError('custom_formatter is not subclass of logging.Formatter', custom_formatter)

    ENABLE_JSON_LOGGING_DEBUG and _logger.info("init framework " + str(framework_name))

    if framework_name:
        framework_name = framework_name.lower()
        if framework_name not in _framework_support_map.keys():
            raise RuntimeError(framework_name + " is not a supported framework")

        _current_framework = _framework_support_map[framework_name]
        global _request_util
        _request_util = util.RequestUtil(request_adapter_class=_current_framework['request_adapter_class'],
                                         response_adapter_class=_current_framework['response_adapter_class'])

        if ENABLE_JSON_LOGGING and _current_framework['app_configurator'] is not None:
            _current_framework['app_configurator']().config()

        _default_formatter = custom_formatter if custom_formatter else JSONLogWebFormatter
    else:
        _default_formatter = custom_formatter if custom_formatter else JSONLogFormatter

    if not enable_json and not ENABLE_JSON_LOGGING:
        _logger.warning(
            "JSON format is not enable, normal log will be in plain text but request logging still in JSON format! "
            "To enable set ENABLE_JSON_LOGGING env var to either one of following values: ['true', '1', 'y', 'yes']")
    else:
        ENABLE_JSON_LOGGING = True
        logging._defaultFormatter = _default_formatter()

    # go to all the initialized logger and update it to use JSON formatter
    ENABLE_JSON_LOGGING_DEBUG and _logger.debug("Update all existing logger to using JSONLogFormatter")
    existing_loggers = list(map(logging.getLogger, logging.Logger.manager.loggerDict))
    util.update_formatter_for_loggers(existing_loggers, _default_formatter)


def init_request_instrument(app=None, custom_formatter=None):
    """
    Configure the request instrumentation logging configuration for given web app. Must be called after init method

    If **custom_formatter** is passed, it will use this formatter over the default.

    :param app: current web application instance
    :param custom_formatter: formatter to override default JSONRequestLogFormatter.
    """

    if _current_framework is None or _current_framework == '-':
        raise RuntimeError("please init the logging first, call init(framework_name) first")

    if custom_formatter:
        if not issubclass(custom_formatter, logging.Formatter):
            raise ValueError('custom_formatter is not subclass of logging.Formatter', custom_formatter)

    configurator = _current_framework['app_request_instrumentation_configurator']()
    configurator.config(app)

    formatter = custom_formatter if custom_formatter else JSONRequestLogFormatter
    request_logger = configurator.request_logger
    request_logger.setLevel(logging.DEBUG)
    request_logger.addHandler(logging.StreamHandler(sys.stdout))
    util.update_formatter_for_loggers([request_logger], formatter)
    request_logger.parent = None


def get_request_logger():
    global _request_logger

    if _current_framework is None or _current_framework == '-':
        raise RuntimeError(
            "request_logger is only available if json_logging is inited with a web app, "
            "call init_<framework_name>() to do that")

    instance = _current_framework['app_request_instrumentation_configurator']._instance
    if instance is None:
        raise RuntimeError("please init request instrument first, call init_request_instrument(app) to do that")

    return instance.request_logger


class RequestInfo(dict):
    """
        class that keep HTTP request information for request instrumentation logging
    """

    def __init__(self, request, **kwargs):
        super(RequestInfo, self).__init__(**kwargs)
        utcnow = datetime.utcnow()
        self.request_start = utcnow
        self.request = request
        self.request_received_at = util.iso_time_format(utcnow)

    # noinspection PyAttributeOutsideInit
    def update_response_status(self, response):
        """
        update response information into this object, must be called before invoke request logging statement
        :param response:
        """
        response_adapter = _request_util.response_adapter
        utcnow = datetime.utcnow()
        time_delta = utcnow - self.request_start
        self.response_time_ms = int(time_delta.total_seconds()) * 1000 + int(time_delta.microseconds / 1000)
        self.response_status = response_adapter.get_status_code(response)
        self.response_size_b = response_adapter.get_response_size(response)
        self.response_content_type = response_adapter.get_content_type(response)
        self.response_sent_at = util.iso_time_format(utcnow)


class BaseJSONFormatter(logging.Formatter):
    """
       Base class for JSON formatters
    """
    base_object_common = {}

    def __init__(self, *args, **kw):
        super(BaseJSONFormatter, self).__init__(*args, **kw)
        if COMPONENT_ID and COMPONENT_ID != EMPTY_VALUE:
            self.base_object_common["component_id"] = COMPONENT_ID
        if COMPONENT_NAME and COMPONENT_NAME != EMPTY_VALUE:
            self.base_object_common["component_name"] = COMPONENT_NAME
        if COMPONENT_INSTANCE_INDEX and COMPONENT_INSTANCE_INDEX != EMPTY_VALUE:
            self.base_object_common["component_instance_idx"] = COMPONENT_INSTANCE_INDEX

    def format(self, record):
        log_object = self._format_log_object(record, request_util=_request_util)
        return JSON_SERIALIZER(log_object)

    def _format_log_object(self, record, request_util):
        utcnow = datetime.utcnow()
        base_obj = {
            "written_at": util.iso_time_format(utcnow),
            "written_ts": util.epoch_nano_second(utcnow),
        }
        base_obj.update(self.base_object_common)
        return base_obj


class JSONRequestLogFormatter(BaseJSONFormatter):
    """
       Formatter for HTTP request instrumentation logging
    """

    def _format_log_object(self, record, request_util):
        json_log_object = super(JSONRequestLogFormatter, self)._format_log_object(record, request_util)
        request = record.request_info.request
        request_adapter = request_util.request_adapter

        length = request_adapter.get_content_length(request)
        json_log_object.update({
            "type": "request",
            "correlation_id": request_util.get_correlation_id(request),
            "remote_user": request_adapter.get_remote_user(request),
            "request": request_adapter.get_path(request),
            "referer": request_adapter.get_http_header(request, 'referer', EMPTY_VALUE),
            "x_forwarded_for": request_adapter.get_http_header(request, 'x-forwarded-for', EMPTY_VALUE),
            "protocol": request_adapter.get_protocol(request),
            "method": request_adapter.get_method(request),
            "remote_ip": request_adapter.get_remote_ip(request),
            "request_size_b": util.parse_int(length, -1),
            "remote_host": request_adapter.get_remote_ip(request),
            "remote_port": request_adapter.get_remote_port(request),
            "request_received_at": record.request_info.request_received_at,
            "response_time_ms": record.request_info.response_time_ms,
            "response_status": record.request_info.response_status,
            "response_size_b": record.request_info.response_size_b,
            "response_content_type": record.request_info.response_content_type,
            "response_sent_at": record.request_info.response_sent_at
        })
        return json_log_object


def _sanitize_log_msg(record):
    return record.getMessage().replace('\n', '_').replace('\r', '_').replace('\t', '_')


class JSONLogFormatter(BaseJSONFormatter):
    """
    Formatter for non-web application log
    """

    def get_exc_fields(self, record):
        if record.exc_info:
            exc_info = self.format_exception(record.exc_info)
        else:
            exc_info = record.exc_text
        return {
            'exc_info': exc_info,
            'filename': record.filename,
        }

    @classmethod
    def format_exception(cls, exc_info):
        return ''.join(traceback.format_exception(*exc_info)) if exc_info else ''

    def _format_log_object(self, record, request_util):
        json_log_object = super(JSONLogFormatter, self)._format_log_object(record, request_util)
        json_log_object.update({
            "msg": _sanitize_log_msg(record),
            "type": "log",
            "logger": record.name,
            "thread": record.threadName,
            "level": record.levelname,
            "module": record.module,
            "line_no": record.lineno,
        })
        if hasattr(record, 'props'):
            json_log_object.update(record.props)

        if record.exc_info or record.exc_text:
            json_log_object.update(self.get_exc_fields(record))

        return json_log_object


class JSONLogWebFormatter(JSONLogFormatter):
    """
    Formatter for web application log
    """

    def _format_log_object(self, record, request_util):
        json_log_object = super(JSONLogWebFormatter, self)._format_log_object(record, request_util)
        json_log_object.update({
            "correlation_id": request_util.get_correlation_id(within_formatter=True),
        })
        return json_log_object


# register flask support
# noinspection PyPep8
import json_logging.framework.flask as flask_support

register_framework_support('flask', None, flask_support.FlaskAppRequestInstrumentationConfigurator,
                           flask_support.FlaskRequestAdapter,
                           flask_support.FlaskResponseAdapter)


def init_flask(custom_formatter=None, enable_json=False):
    __init(framework_name='flask', custom_formatter=custom_formatter, enable_json=enable_json)


# register sanic support
# noinspection PyPep8
from json_logging.framework.sanic import SanicAppConfigurator, SanicAppRequestInstrumentationConfigurator, \
    SanicRequestAdapter, SanicResponseAdapter

register_framework_support('sanic', SanicAppConfigurator,
                           SanicAppRequestInstrumentationConfigurator,
                           SanicRequestAdapter,
                           SanicResponseAdapter)


def init_sanic(custom_formatter=None, enable_json=False):
    __init(framework_name='sanic', custom_formatter=custom_formatter, enable_json=enable_json)


# register quart support
# noinspection PyPep8
import json_logging.framework.quart as quart_support

register_framework_support('quart', None, quart_support.QuartAppRequestInstrumentationConfigurator,
                           quart_support.QuartRequestAdapter,
                           quart_support.QuartResponseAdapter)


def init_quart(custom_formatter=None, enable_json=False):
    __init(framework_name='quart', custom_formatter=custom_formatter, enable_json=enable_json)


# register connexion support
# noinspection PyPep8
import json_logging.framework.connexion as connexion_support

register_framework_support('connexion', None, connexion_support.ConnexionAppRequestInstrumentationConfigurator,
                           connexion_support.ConnexionRequestAdapter,
                           connexion_support.ConnexionResponseAdapter)


def init_connexion(custom_formatter=None, enable_json=False):
    __init(framework_name='connexion', custom_formatter=custom_formatter, enable_json=enable_json)
